package soda;

import com.wazeegroup.physhun.framework.StateModel;
import com.wazeegroup.physhun.engine.SimpleProcessContainer;
import com.wazeegroup.physhun.engine.StandardEngine;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import org.springframework.context.support.ClassPathXmlApplicationContext;
import soda.events.FundsAdded;
import soda.events.SelectionButtonPressed;
import soda.events.CoinReturnPressed;

/**
 * @author Justin McCarter
 *         Date: Feb 24, 2006
 *         Copyright 2006 Wazee Group, LLC
 *         Licensed under the Apache License, Version 2.0 (the "License");
 *         you may not use this file except in compliance with the License.
 *         You may obtain a copy of the License at
 *         <p/>
 *         http://www.apache.org/licenses/LICENSE-2.0
 *         <p/>
 *         Unless required by applicable law or agreed to in writing, software
 *         distributed under the License is distributed on an "AS IS" BASIS,
 *         WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *         See the License for the specific language governing permissions and
 *         limitations under the License.
 */
public class VirtualSodaMachineUI extends JFrame {
    private SimpleProcessContainer container;
    private StateModel stateModel;
    private VirtualSodaSodaTransaction currentTransaction;
    private JLabel fundsLabel;
    private JLabel messageLabel;

    public SodaTransaction getCurrentTransaction() {
        if (currentTransaction == null
                || stateModel.isProcessComplete(currentTransaction)) {

            //create and start a new transaction
            currentTransaction = new VirtualSodaSodaTransaction("CurrentTransaction");
            currentTransaction.addChangeListener(new PropertyChangeListener() {
                public void propertyChange(PropertyChangeEvent evt) {
                    fundsLabel.setText("$" + currentTransaction.getFunds());
                    messageLabel.setText(currentTransaction.getLastMessage());
                }
            });
            container.startProcess(currentTransaction, stateModel);
        }
        return currentTransaction;
    }
    public VirtualSodaMachineUI() {
        super("Virtual Soda Machine");
        setUpUI();
    }

    private void setUpUI() {
        container = new SimpleProcessContainer();
        container.setProcessEngine(new StandardEngine());
        ClassPathXmlApplicationContext ctx =
                new ClassPathXmlApplicationContext(new String[]{"sodaModel.xml"});
        stateModel = (StateModel) ctx.getBean("stateModel");  //retrieve the stateModel bean.
        setLayout(new GridLayout(5, 2));
        JButton cokeButton = new JButton("Coke");
        JButton dietCokeButton = new JButton("Diet Coke");
        JButton rootBeerButton = new JButton("Root Beer");
        JButton skipperButton = new JButton("The Skipper");
        JButton orangeCrushButton = new JButton("Orange Crush");
        messageLabel = new JLabel("Ice Cold Drinks");
        fundsLabel = new JLabel("0.00");
        final JTextField moneyInputField = new JTextField();
        JButton moneyInputButton = new JButton("Add money");
        JButton coinReturnButton = new JButton("Coin Return");
        add(cokeButton);
        add(fundsLabel);
        add(dietCokeButton);
        add(messageLabel);
        add(rootBeerButton);
        add(moneyInputField);
        add(skipperButton);
        add(moneyInputButton);
        add(orangeCrushButton);
        add(coinReturnButton);
        pack();
        ActionListener buttonPressActionListener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JButton sourceButton = (JButton)e.getSource();
                container.sendTriggerEventToProcess(getCurrentTransaction().getID(), new SelectionButtonPressed(sourceButton.getText()));
            }
        };
        cokeButton.addActionListener(buttonPressActionListener);
        dietCokeButton.addActionListener(buttonPressActionListener);
        rootBeerButton.addActionListener(buttonPressActionListener);
        skipperButton.addActionListener(buttonPressActionListener);
        orangeCrushButton.addActionListener(buttonPressActionListener);
        coinReturnButton.addActionListener(
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        container.sendTriggerEventToProcess(getCurrentTransaction().getID(),
                                new CoinReturnPressed());
                    }
                }
        );
        moneyInputButton.addActionListener(
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        try {
                            float amount = Float.parseFloat(moneyInputField.getText());
                            container.sendTriggerEventToProcess(getCurrentTransaction().getID(),
                                    new FundsAdded(amount));
                            moneyInputField.setText("");
                        }
                        catch (NumberFormatException nfe) {
                            moneyInputField.setText("");
                        }
                    }
                }
        );
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                super.windowClosing(e);
                System.exit(0);
            }
        });
    }

    public static void main(String[] args) {
        VirtualSodaMachineUI ui = new VirtualSodaMachineUI();
        ui.setVisible(true);
    }
}
